import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/src/models/comment.dart';
import 'package:ocg_post/src/repository/comment/base.dart';

abstract class CommentListEvent {}

class CommentListRefreshEvent extends CommentListEvent {
  final Comment after;
  final String postId;
  final int count;

  CommentListRefreshEvent(this.after, this.postId, this.count);
}

class CommentListAddEvent extends CommentListEvent {
  final Comment comment;

  CommentListAddEvent(this.comment);
}

class CommentListDeletedEvent extends CommentListEvent {
  final Comment comment;

  CommentListDeletedEvent(this.comment);
}

abstract class CommentListState {}

class CommentListInitialState extends CommentListState {}

class CommentListLoadingState extends CommentListState {
  final List<Comment> comments;

  CommentListLoadingState(this.comments);
}

class CommentListLoadedState extends CommentListState {
  final List<Comment> comments;
  CommentListLoadedState(this.comments);
}

class CommentListErrorState extends CommentListState {
  final String error;
  CommentListErrorState(this.error);
}

class CommentListBloc extends Bloc<CommentListEvent, CommentListState> {
  final CommentRepository commentRepository;
  Set<Comment> _comments = Set<Comment>();
  CommentListBloc(this.commentRepository) : super(CommentListInitialState());
  @override
  Stream<CommentListState> mapEventToState(CommentListEvent event) async* {
    if (event is CommentListRefreshEvent) {
      yield CommentListLoadingState(_comments.toList());
      if (event.after == null)
        _comments = _comments
            .where((element) => element.postId != event.postId)
            .toSet();
      AppResponse<List<Comment>> response =
          await commentRepository.getPage(event.after, event.postId, 10);
      if (response.isSuccess) {
        if (response.isSuccess) {
          _comments.addAll(response.data);
        }
        yield CommentListLoadedState(_comments.toList());
      } else {
        yield CommentListErrorState(response.error);
      }
    }
    if (event is CommentListAddEvent) {
      if (_comments.contains(event.comment)) {
        _comments.remove(event.comment);
      }
      _comments.add(event.comment);
      yield CommentListLoadedState(_comments.toList());
    }
    if (event is CommentListDeletedEvent) {
      _comments.remove(event.comment);
      yield CommentListLoadedState(_comments.toList());
    }
  }
}
