import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/src/models/hashtag.dart';
import 'package:ocg_post/src/repository/hashtag/base.dart';

abstract class HashtagListEvent {}

class HashtagListRefreshEvent extends HashtagListEvent {}

abstract class HashtagListState {}

class HashtagListInitialState extends HashtagListState {}

class HashtagListLoadingState extends HashtagListState {}

class HashtagListLoadedState extends HashtagListState {
  final List<Hashtag> hashtags;
  HashtagListLoadedState(this.hashtags);
}

class HashtagListErrorState extends HashtagListState {
  final String error;
  HashtagListErrorState(this.error);
}

class HashtagListBloc extends Bloc<HashtagListEvent, HashtagListState> {
  final HashtagRepository hashtagRepository;
  HashtagListBloc(this.hashtagRepository) : super(HashtagListInitialState());
  @override
  Stream<HashtagListState> mapEventToState(HashtagListEvent event) async* {
    if (event is HashtagListRefreshEvent) {
      yield HashtagListLoadingState();
      AppResponse<List<Hashtag>> response = await hashtagRepository.getList();
      if (response.isSuccess) {
        yield HashtagListLoadedState(response.data);
      } else {
        yield HashtagListErrorState(response.error);
      }
    }
  }
}
