import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/src/models/post.dart';
import 'package:ocg_post/src/repository/post/base.dart';

abstract class PostCreateEvent {}

class PostCreateStartEvent extends PostCreateEvent {
  final Post post;
  PostCreateStartEvent(this.post);
}

abstract class PostCreateState {}

class PostCreateIdleState extends PostCreateState {}

class PostCreateLoadingState extends PostCreateState {}

class PostCreateSuccessState extends PostCreateState {
  PostCreateSuccessState();
}

class PostCreateErrorState extends PostCreateState {
  final String error;
  PostCreateErrorState(this.error);
}

class PostCreateBloc extends Bloc<PostCreateEvent, PostCreateState> {
  final PostRepository postRepository;
  PostCreateBloc(this.postRepository) : super(PostCreateIdleState());
  @override
  Stream<PostCreateState> mapEventToState(PostCreateEvent event) async* {
    if (event is PostCreateStartEvent) {
      yield PostCreateLoadingState();
      AppResponse<String> response = await postRepository.create(event.post);
      if (response.isSuccess) {
        yield PostCreateSuccessState();
      } else {
        yield PostCreateErrorState(response.error);
      }
    }
  }
}
