import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_post/ocg_post.dart';

abstract class PostDetailEvent {}

class PostDetailRefreshEvent extends PostDetailEvent {
  final String postId;

  PostDetailRefreshEvent(this.postId);
}

abstract class PostDetailState {}

class PostDetailSuccessState extends PostDetailState {
  final Post post;
  final List<Comment> comments;

  PostDetailSuccessState(this.post, this.comments);

  PostDetailSuccessState.named({this.post, this.comments});
}

class PostDetailLoadingState extends PostDetailState {
  final String id;

  PostDetailLoadingState(this.id);
}

class PostDetailErrorState extends PostDetailState {
  final String id;
  final String error;

  PostDetailErrorState(this.id, this.error);
}

class PostDetailInitialState extends PostDetailState {}

class PostDetailBloc extends Bloc<PostDetailEvent, PostDetailState> {
  final PostRepository repository;
  final CommentRepository commentRepository;
  PostDetailBloc(this.repository, this.commentRepository)
      : super(PostDetailInitialState());

  @override
  Stream<PostDetailState> mapEventToState(PostDetailEvent event) async* {
    if (event is PostDetailRefreshEvent) {
      yield PostDetailLoadingState(event.postId);
      Post post;
      List<Comment> comments = [];
      await Future.wait([
        Future(() async {
          var response = await repository.read(event.postId);
          post = response.data;
        }),
        Future(() async {
          var response =
              await commentRepository.getPage(null, event.postId, 10);
          comments = response.data ?? [];
        })
      ]);
      if (post != null) {
        yield PostDetailSuccessState(post, comments);
      } else {
        yield PostDetailErrorState(event.postId, "Post not found");
      }
    }
  }
}
