import 'package:flutter/cupertino.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/ocg_post.dart';
import 'package:ocg_post/src/repository/post/base.dart';

abstract class PostUpdateEvent {}

class PostUpdateReactEvent extends PostUpdateEvent {
  final String postId;
  final UserReaction reaction;

  PostUpdateReactEvent(this.postId, this.reaction);
}

abstract class PostUpdateState {}

class PostUpdateLoadingState extends PostUpdateState {
  final PostUpdateEvent event;

  PostUpdateLoadingState(this.event);
}

class PostUpdateSuccessState extends PostUpdateState {
  final PostUpdateEvent event;

  PostUpdateSuccessState(this.event);
}

class PostUpdateErrorState extends PostUpdateState {
  final PostUpdateEvent event;
  final String error;

  PostUpdateErrorState(this.event, this.error);
}

class PostUpdateInitialState extends PostUpdateState {}

class PostUpdateBloc extends Bloc<PostUpdateEvent, PostUpdateState> {
  final PostRepository postRepository;
  PostUpdateBloc(this.postRepository) : super(PostUpdateInitialState());

  @override
  Stream<PostUpdateState> mapEventToState(PostUpdateEvent event) async* {
    if (event is PostUpdateReactEvent) {
      debugPrint("Post React ${event.postId}");
      yield PostUpdateLoadingState(event);
      AppResponse<bool> response =
          await postRepository.reactToPost(event.postId, event.reaction);
      if (response.isSuccess) {
        yield PostUpdateSuccessState(event);
      } else {
        yield PostUpdateErrorState(event, response.error);
      }
    }
  }
}
