import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/ocg_post.dart';

abstract class ReactionListEvent {}

class ReactionListRefreshEvent extends ReactionListEvent {
  final String postId;

  ReactionListRefreshEvent(
    this.postId,
  );
}

abstract class ReactionListState {}

class ReactionListInitialState extends ReactionListState {}

class ReactionListLoadingState extends ReactionListState {}

class ReactionListLoadedState extends ReactionListState {
  final String postId;
  final List<UserReaction> reactions;
  final List<PostReaction> postReactions;
  ReactionListLoadedState(this.postId, this.reactions, this.postReactions);
}

class ReactionListErrorState extends ReactionListState {
  final String error;
  ReactionListErrorState(this.error);
}

class ReactionListBloc extends Bloc<ReactionListEvent, ReactionListState> {
  final PostReactionRepository reactionRepository;
  ReactionListBloc(this.reactionRepository) : super(ReactionListInitialState());
  @override
  Stream<ReactionListState> mapEventToState(ReactionListEvent event) async* {
    if (event is ReactionListRefreshEvent) {
      yield ReactionListLoadingState();
      String postId = event.postId;
      List<UserReaction> userReactions = [];
      List<PostReaction> postReactions = [];
      await Future.wait([
        Future(() async {
          AppResponse<List<PostReaction>> response =
              await reactionRepository.getPostReactions(
            postId,
          );
          postReactions = response.data ?? [];
        }),
        Future(() async {
          AppResponse<List> response =
              await reactionRepository.getReactions(postId);
          userReactions = response.data ?? [];
        })
      ]);
      yield ReactionListLoadedState(postId, userReactions, postReactions);
    }
  }
}
