import 'package:equatable/equatable.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_post/ocg_post.dart';
import 'package:ocg_profile/ocg_profile.dart';

part 'post.g.dart';

@JsonSerializable(explicitToJson: true)
class Post extends Equatable {
  final String id;
  @JsonKey(
    toJson: convertObject,
  )
  final Profile profile;
  final String description;
  final List<PostData> data;
  final Address taggedLocation;
  final List<Hashtag> hashtags;
  final List<PostReaction> reactions;
  final List<UserReaction> myReactions;
  final int reactionCount;
  final DateTime recordedAt;
  final List<Profile> taggedPeople;
  final bool isSelf;
  final List<Comment> topComments;

  Post(
      this.id,
      this.profile,
      this.description,
      this.data,
      this.taggedLocation,
      this.myReactions,
      this.recordedAt,
      this.hashtags,
      this.reactions,
      this.reactionCount,
      this.taggedPeople,
      this.isSelf,
      this.topComments);

  Post.named(
      {this.id,
      this.profile,
      this.description,
      this.data,
      this.taggedLocation,
      this.reactions,
      this.myReactions,
      this.reactionCount,
      this.hashtags,
      this.recordedAt,
      this.taggedPeople,
      this.isSelf,
      this.topComments});

  static String convertObject(profile) {
    return profile.id;
  }

  Post copyWith(
      {String id,
      Profile profile,
      String description,
      List<PostData> data,
      Address taggedLocation,
      List<Profile> taggedPeople,
      List<PostReaction> reactions,
      int reactionCount,
      DateTime recordedAt,
      List<Hashtag> hashtags,
      List<UserReaction> myReactions,
      bool isSelf,
      List<Comment> topComments}) {
    return Post.named(
        id: id ?? this.id,
        profile: profile ?? this.profile,
        description: description ?? this.description,
        data: data ?? this.data,
        taggedLocation: taggedLocation ?? this.taggedLocation,
        reactions: reactions ?? this.reactions,
        reactionCount: reactionCount ?? this.reactionCount,
        recordedAt: recordedAt ?? this.recordedAt,
        hashtags: hashtags ?? this.hashtags,
        taggedPeople: taggedPeople ?? this.taggedPeople,
        myReactions: myReactions ?? this.myReactions,
        isSelf: isSelf ?? this.isSelf,
        topComments: topComments ?? this.topComments);
  }

  Map<String, dynamic> toJson() {
    var json = _$PostToJson(this);
    return json;
  }

  factory Post.fromJson(json) => _$PostFromJson(json);

  @override
  List<Object> get props => [id];
}

class PostDataType {
  static const String IMAGE = "image";
  static const String VIDEO = "video";
}

@JsonSerializable(explicitToJson: true)
class PostData {
  final String url;
  final String type;
  final Map<String, dynamic> metadata;

  PostData(this.url, this.type, this.metadata);

  PostData.named({this.url, this.type, this.metadata});

  Map<String, dynamic> toJson() => _$PostDataToJson(this);

  factory PostData.fromJson(json) => _$PostDataFromJson(json);
}

class Reaction {
  final String type;

  Reaction(
    this.type,
  );

  Reaction.named({
    this.type,
  });
}

@JsonSerializable(explicitToJson: true)
class PostReaction extends Reaction {
  final int count;

  PostReaction(String type, this.count) : super(type);

  PostReaction.named({String type, this.count}) : super(type);

  Map<String, dynamic> toJson() => _$PostReactionToJson(this);

  factory PostReaction.fromJson(json) => _$PostReactionFromJson(json);

  PostReaction copyWith({String type, int count}) {
    return PostReaction.named(
        type: type ?? this.type, count: count ?? this.count);
  }
}

@JsonSerializable(explicitToJson: true)
class UserReaction extends Reaction {
  final String id;
  final Profile profile;
  final DateTime recordedAt;

  UserReaction(this.id, String type, this.profile, this.recordedAt)
      : super(type);

  UserReaction.named({this.id, String type, this.profile, this.recordedAt})
      : super(type);

  Map<String, dynamic> toJson() => _$UserReactionToJson(this);

  UserReaction copyWith(
      {String id, String type, Profile profile, DateTime recordedAt}) {
    return UserReaction.named(
        id: id ?? this.id,
        profile: profile ?? this.profile,
        recordedAt: recordedAt ?? this.recordedAt,
        type: type ?? this.type);
  }

  factory UserReaction.fromJson(json) => _$UserReactionFromJson(json);
}
