import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_post/src/models/hashtag.dart';
import 'package:ocg_post/src/repository/hashtag/base.dart';

class HashtagFirebaseRepository extends HashtagRepository {
  final FirebaseFirestore _firestore;
  final IDHelper idHelper;

  static const String hashtags = 'hashtags';

  CollectionReference get hashtagCollection => _firestore.collection(hashtags);

  HashtagFirebaseRepository(this.idHelper)
      : this._firestore = FirebaseFirestore.instance;

  @override
  Future<AppResponse<String>> create(Hashtag object) async {
    try {
      if (InputHelper.isBlank(object.title)) {
        return AppResponse.named(error: "Title is null");
      }
      var snapshot =
          await hashtagCollection.where("title", isEqualTo: object.title).get();
      if (snapshot.docs.length > 0) {
        return AppResponse.named(
            data: Hashtag.fromJson(snapshot.docs[0].data()).id);
      } else {
        object = object.copyWith(
            id: idHelper.generateId(), hash: _generateHash(object.title));
        await hashtagCollection.doc(object.id).set(object.toJson());
        return AppResponse.named(data: object.id);
      }
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  String _generateHash(String hashtag) {
    return hashtag
        .trim()
        .replaceAll("#", "")
        .replaceAll("", "")
        .replaceAll(".", "")
        .toLowerCase();
  }

  @override
  Future<AppResponse<bool>> delete(String id) {
    // TODO: implement delete
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<Hashtag>>> getList(
      {int page = 1, int count = 10}) async {
    try {
      var snapshot = await hashtagCollection.limit(count * page).get();
      var documents =
          snapshot.docs.map((e) => e.data()).skip((page - 1) * count).toList();
      var list = (await Future.wait(documents.map((e) async {
        return Hashtag.fromJson(e);
      }).toList()))
          .where((element) => element != null)
          .toList();
      return AppResponse.named(data: list);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<Hashtag>> read(String id) async {
    try {
      var snapshot = await hashtagCollection.doc(id).get();
      if (snapshot.exists) {
        return AppResponse.named(data: Hashtag.fromJson(snapshot.data()));
      } else {
        return AppResponse.named(error: "Not exists");
      }
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> update(Hashtag object) {
    // TODO: implement update
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<Hashtag>>> getPage(Hashtag object, int count) {
    // TODO: implement getPage
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<Hashtag>> getByIdentifier(String hashtag) async {
    try {
      var snapshot = await hashtagCollection
          .where("title", isEqualTo: hashtag)
          .limit(1)
          .get();
      if (snapshot.docs.length > 0) {
        return AppResponse.named(
            data: Hashtag.fromJson(snapshot.docs[0].data()));
      }
      return AppResponse.named(error: "Not Found");
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }
}
